#ifndef _LIKELIHOOD_H_
#define _LIKELIHOOD_H_

#include <iostream>
#include <cmath>
#include <fstream>
#include <sstream>
#include <vector>
#include <string>

#include "TROOT.h"
#include "TTree.h"
#include "TFile.h"
#include "TString.h"

#include "Config.h"
#include "TDC.h"

class Likelihood {
private:
   TFile *rootfile;
   TTree *tr;
   float tr_lnL;
   int tr_n;
   TDC *pdf;

   int evt_min , evt_max;
   std::vector<float> norm;
   std::string tree_name;
   std::vector< bool >veto_ch_flag;

   TH1F *h_l , *h_nhit;
   std::vector< TH1F* > h_lch;

   int tdc_min , tdc_max;

public:
   Likelihood( void );
   ~Likelihood( void ){};

   //Generals
   void SetEventRange( int min , int max ){ evt_min = min ;evt_max = max; };
   void SetEventRange( float min , float max ){ evt_min = min ;evt_max = max; }; //ratio of event , evt_max < 0
   void SetTdcRange( int min , int max ){ tdc_min = min;tdc_max = max; };
   void SetNormFactor( float n , int ch = -1); // ch = -1 : allch

   float GetSumPdf( int ch = -1 , int start_bin = 0 , int end_bin = 4000 ); //ch=-1 : all ch
   float GetMean( int ch = -1 ){ if( ch > 0 ){ return h_lch[ch]->GetMean(); }else{ return h_l->GetMean(); }};
   float GetEntries( int ch = -1 ){ if( ch > 0 ){ return h_lch[ch]->GetEntries(); }else{ return h_l->GetEntries(); }};
   float GetRMS( int ch = -1 ){ if( ch > 0 ){ return h_lch[ch]->GetRMS(); }else{ return h_l->GetRMS(); }};
   float GetNhitMean( void ){ return h_nhit->GetMean();};
   float GetNhitEntries( void ){ return h_nhit->GetEntries();};
   float GetNhitRMS( void ){ return h_nhit->GetRMS();};

   void Draw( int ch = -1 ){ if( ch > 0 ){ h_lch[ch]->Draw(); }else{ h_l->Draw(); }};
   void DrawNhit( void ){ h_nhit->Draw(); };

   void Clear( void );
   void Close( void );

   void Write( std::string fname );

   //Set Files
   void SetTreeName( std::string tree ){ tree_name = tree; };

   void SetRootFile( std::string fname ){ rootfile = new TFile( (TString)fname );};
   void SetRootFile( TFile *file ){ rootfile = file;};

   void SetPdfFile( std::string pdffile ){ pdf->SetPdfFile( pdffile ); };
   void SetPdfFile( TDC *p ){ pdf = p; };

   //Calc likelihood
   void Calc( void );
   //void Calc_MC( void );
   //void CalcOldMC( void );

   //GetLikelihood value 
   float GetLikelihood( void );

};

#endif //_LIKELIHOD_H_
